(function (glb) {
    /**模块映射表 */
    var _context = {},
        _commentRegExp = /\/\*[\s\S]*?\*\/|([^:"'=]|^)\/\/.*$/mg,
        _cjsRequireRegExp = /[^.]\s*require\s*\(\s*["']([^'"\s]+)["']\s*\)/g,
        _cfx = {
            ONLYJS: true
        },
        //如果define执行是不能获取模块名，暂时保存依赖和模块代码，后续onload时获取名称，加载模块依赖
        _currentDefine = {},
        _package = '',
        currentlyAddingScript = '';
    var doc = document;
    var head = doc.head || doc.getElementsByTagName("head")[0] || doc.documentElement;
    var baseElement = head.getElementsByTagName("base")[0];


    var _ROOT = 0;
    var _ENV = {
        development: 'development',
        production: 'production'
    }
    var util = (function () {
    function log(msg) {
        if (_cfx.env == _ENV.development) {
            console.log(msg);
        }
    }

    function each(array, cb) {
        if (!isAry(array)) {
            for (var pro in array) {
                if (!cb)
                    break;

                if (cb.call(array[pro], array[pro], pro) === false) {
                    break;
                }
            }
        } else {
            for (var i = 0; i < array.length; i++) {
                if (!cb)
                    break;

                if (cb.call(array[i], array[i], i) === false) {
                    break;
                }
            }
        }
    }
    function assign(target, src) {
        if (!src) {
            return target;
        }
        if (!target) {
            target = {};
        }
        for (var pro in src) {
            target[pro] = src[pro];
        }
        return target;
    }

    function isType(type) {
        return function (obj) {
            return {}.toString.call(obj) == "[object " + type + "]";
        }
    }

    var isObj = isType("Object");
    var isStr = isType("String");
    var isAry = Array.isArray || isType("Array");
    var isFunc = isType("Function");
    var isUndef = isType("Undefined");

    function _getBaseUrl() {
        var doc = document;
        var head = doc.head || doc.getElementsByTagName("head")[0] || doc.documentElement;
        var baseElement = head.getElementsByTagName("base")[0];
        if (baseElement) {
            var url = baseElement.href;
            if (url.charAt(url.length - 1) !== '/') {
                url += '/';
            }
            return url;
        }
       
        var regx = new RegExp('(http|https|file)://([^\/]*\/)*');
        var val = regx.exec(window.location.href);
        return val[0];
    }

    function _bind(func, context) {
        return function () {
            return func.apply(context, arguments);
        }
    }

    return {
        log: log,
        assign: assign,
        each: each,
        isObject:isObj,
        isString: isStr,
        isArray: isAry,
        isFunction: isFunc,
        isUndefined: isUndef,
        getBaseUrl: _getBaseUrl,
        bind: _bind       
    }
})();
;
    var storage = (function () {

    var ls = window.localStorage;
    var sptLoc = true;
    return {
        get: function (key, cb) {
            if (ls && sptLoc) {
                try {
                    cb && cb(ls.getItem(key));
                } catch (ex) {
                    sptLoc = false;
                    cb && cb(null);
                }
            } else {
                cb && cb(null);
            }
        },
        set: function (key, v) {
            if (ls && sptLoc) {
                try {                    
                    ls.setItem(key, v);
                } catch (ex) {
                    sptLoc = false;
                }
            }
        },
        remove: function (key) {
            if (ls) {
                ls.removeItem(key);
            }
        }
    }
})();
    var loaderManager = (function () {
    //存储loader实现
    var _context = {},
        _store = {},
        _extRegx = /\.(\w*)$/;

        
    function addCssToDom(cssText) {
        var style = doc.createElement('style');
        style.type='text/css';
        style.innerHTML=cssText;
        // ref: #185 & http://dev.jquery.com/ticket/2709
        baseElement ?
            head.insertBefore(style, baseElement) :
            head.appendChild(style)
    }
    var jsLoader = {
        addOnload: function (node, cb, url) {
            var supportOnload = "onload" in node

            if (supportOnload) {
                node.onload = onload
                node.onerror = function () {
                    // emit("error", { uri: url, node: node })
                    onload(true)
                }
            }
            else {
                node.onreadystatechange = function () {
                    if (/loaded|complete/.test(node.readyState)) {
                        onload()
                    }
                }
            }

            function onload(error) {
                // Ensure only run once and handle memory leak in IE
                node.onload = node.onerror = node.onreadystatechange = null

                // Remove the script to reduce memory leak
                // if (!data.debug) {
                head.removeChild(node)
                // }

                // Dereference the node
                node = null

                cb(error)
            }
        },
        load: function (url, cb, charset, crossorigin) {
            var node = doc.createElement("script")
            if (charset) {
                node.charset = charset
            }
            if (!util.isUndefined(crossorigin)) {
                node.setAttribute("crossorigin", crossorigin)
            }
            this.addOnload(node, cb, url)

            node.async = true
            node.src = url

            // For some cache cases in IE 6-8, the script executes IMMEDIATELY after
            // the end of the insert execution, so use `currentlyAddingScript` to
            // hold current node, for deriving url in `define` call
            currentlyAddingScript = node

            // ref: #185 & http://dev.jquery.com/ticket/2709
            baseElement ?
                head.insertBefore(node, baseElement) :
                head.appendChild(node)

            currentlyAddingScript = null
        },
        exec: function (mod) {
            var exports = {};
            var req = util.bind(mod.require, mod);
            req.async = glb.require.async;
            req.style = addCssToDom;
            if(!mod.factory){
                util.log(mod.name+' factory is undefined');
                return;
            }
            var result = mod.factory.apply(_context, [req, exports, mod]);
            if (result) {
                return result;
            } else {
                return util.assign(mod.exports, exports);
            }
        }
    }

    var cssLoader = {
        addToDom: function (css) {
            var link = doc.createElement('link');
            link.href = url;
            link.rel = 'stylesheet';
            link.type = 'text/css';
            link.innerHTML = css;
            link.addEventListener('load', function (a) {
                cb && cb(false, url);
            })
            // ref: #185 & http://dev.jquery.com/ticket/2709
            baseElement ?
                head.insertBefore(link, baseElement) :
                head.appendChild(link)
        },
        load: function (url, cb) {
            var link = doc.createElement('link');
            link.href = url;
            link.rel = 'stylesheet';
            link.type = 'text/css';
            link.addEventListener('load', function (a) {
                cb && cb(false, url);
            })
            // ref: #185 & http://dev.jquery.com/ticket/2709
            baseElement ?
                head.insertBefore(link, baseElement) :
                head.appendChild(link)
        },
        exec: function (mod) {
            if (!mod.factory) {
                return;
            }
            var exports = {};
            var req = util.bind(mod.require, mod);
            req.async = glb.require.async;
            var result = mod.factory.apply(_context, [req, exports, mod]);
            if (!result) {
                result = util.assign(mod.exports, exports);
            }
            if (result) {
                this.addToDom(result);
            }
        }
    }

    _store['.js'] = jsLoader;
    _store['.css'] = cssLoader;
    return {
        get: function (name) {
            if (_cfx.ONLYJS) {
                return jsLoader;
            }
            var matchs = _extRegx.exec(name);
            if (matchs && matchs.length > 0) {
                var ext = matchs[0];
                if (ext) {
                    return _store[ext];
                }
            }
            return jsLoader;
        },
        set: function (ext, loader) {
            _store[ext] = loader;
        }
    }
})();
    var loader = (function () {
    /**
     * 模块加载，提供统一loader接口，可以通过在本地存储加载，也可以从网络加载
     */

    var doc = document;
    var head = doc.head || doc.getElementsByTagName("head")[0] || doc.documentElement;
    var baseElement = head.getElementsByTagName("base")[0];

    var reg_def = /^define\(/;
    var reg_defname = /^define\(['"][^'"]*['"]/;
    var reg_pro = /^(http|https|file):\/\/[^\/]*/;

    var baseUrl = _cfx.baseUrl;
    var locStore = {
        getUnid: function (url) {
            //根目录
            return url.replace(reg_pro, '');
        },
        load: function (url, cb) {
            //本地加载，模拟define执行过程
            var key = this.getUnid(url);
            storage.get(key, function (code) {
                if (!code) {
                    cb && cb(true, url);
                    return;
                }
                //是否有define
                if (reg_def.test(code)) {
                    //检查是否有name
                    if (!reg_defname.test(code)) {
                        code = code.replace(/^define\s*\(/, 'define(\'' + key + '\',');
                    }
                    eval(code);
                    cb && cb(false, url);

                } else {
                    code = 'define(\'' + key + '\',' + code + ');';
                    eval(code);
                    cb && cb(false, url);
                }
            });
        },
        save: function (url, factory) {
            var key = this.getUnid(url);
            var code = 'define(\'' + key + '\',' + factory.toString() + ');';
            storage.set(key, code);
        },
        remove: function (url) {
            var key = this.getUnid(url);
            storage.remove(key);
        }
    }

    //本地加载依赖
    var loc = {
        load: function (deps, cb) {
            // var notExist = [];
            util.each(deps, function (dep) {
                var url = dep;
                locStore.load(url, function (error, dep) {
                    if (error === true) {
                        // notExist.push(dep);
                        cb && cb(error, [dep])
                    } else {
                        cb && cb(error, [dep])
                    }
                });
            });
            // if (notExist.length > 0) {
            //     cb && cb(true, notExist);
            // }
        }
    }

    var combo = {
        load: function (deps, loader, cb) {
            var url = '??';
            util.each(deps, function (dep, i) {
                url += removeBase(dep) + ',';
            });
            url = _cfx.comboHandler + url.substring(0, url.length - 1);
            util.log(url);

            loader.load(url, function (error) {
                cb && cb(error, deps);
            })
        }
    }
    var single = {
        load: function (deps, loader, callbck) {
            util.each(deps, function (dep, i) {
                // var url = parseName(dep);
                var url = dep;
                loader.load(url, (function (d) {
                    return function (event) {

                        callbck && callbck(event, d);
                    }
                })(dep));
            })
        }
    }


    var network = {
        load: function (deps, cb) {
            //根据后缀名分拣
            var types = {};
            var extRegx = /\.\w*$/;
            for (var i = 0; i < deps.length; i++) {
                var ext = deps[i].match(extRegx)[0];
                if (!types[ext]) { types[ext] = [] }
                types[ext].push(deps[i]);
            }

            var loadeHandler = _cfx.comboHandler ? combo : single;

            for (var t in types) {
                var resLoader = loaderManager.get(t);

                // (function (resLoader, deps) {
                    loadeHandler.load(deps, resLoader, cb);
                // })(resLoader, types[t])
            }
        }
    }


    var result = {
        load: function (deps, cb) {
            //先看模块是否有本地存储，如果有就不需要加载
            if (_cfx.localStore) {
                function _loadFirstLocal(deps, cb) {

                    loc.load(deps, function (error, deps) {
                        if (error) {
                            if (!deps || deps.length <= 0) {
                                return;
                            }
                            network.load(deps, function (error, dep) {
                                cb && cb(error, dep);
                            });
                        } else {
                            cb && cb(error, deps);
                        }
                    });
                }
                result.load = _loadFirstLocal;
            } else {
                function _loadFirstNet(deps, cb) {

                    network.load(deps, function (error, dep) {
                        cb && cb(error, dep);
                    });
                }
                result.load = _loadFirstNet;
            }

            result.load(deps, cb);
        },
        update: function (deps, cb) {
            network.load(deps, function (error, dep) {
                cb && cb(error, dep);
            });
        },
        save: function (url, factory) {
            if (!factory) { return; }
            if (_cfx.localStore) {
                locStore.save(url, factory);
            }
        },
        remove: function (url) {
            if (_cfx.localStore) {
                locStore.remove(url);
            }
        }
    }
    return result;
})();;
    var Mod = (function () {

    var _reg = {},//name和模块返回结果的映射表
        _map = {};//存储模块加载状态
    var _STATUS = {
        init: 0,
        loading: 2,
        loaded: 4,
        executing: 6,
        executed: 8,
        error: -1
    }

    //模块对象
    function Mod(options) {
        var me = this;
        //模块的名称，标识，如果没有定义，取模块的uri。
        me.name = options.name;

        // this.depsCount = this.deps.length;
        //模块脚本uri
        me.uri = options.uri;
        //模块当前状态
        me.base = options.base;

        me.status = options.status || _STATUS.init;
        //模块的代码
        me.factory = options.factory;
        //存储模块父级模块,如果没有父亲模块，设置为自身
        // this.parent = options.parent || this;

        me.exports = {};

        me.events = {};

        me.depsCount = 0;

        me.remain = {};

        //存储的是Mod实例
        me.deps = options.deps || [];

        // me.loader = loaderManager.get(me.uri);

    }
    Mod.prototype.on = function (name, cb) {
        var me = this;
        var list = me.events[name] || (me.events[name] = [])
        list.push(cb)
        return me;
    }
    Mod.prototype.off = function (name, cb) {
        var me = this;
        // Remove *all* events
        if (!(name || cb)) {
            me.events = {}
            return me;
        }

        var list = me.events[name]
        if (list) {
            if (cb) {
                for (var i = list.length - 1; i >= 0; i--) {
                    if (list[i] === cb) {
                        list.splice(i, 1)
                    }
                }
            }
            else {
                delete events[name]
            }
        }

        return me;
    }

    Mod.prototype.emit = function (name, data) {

        var me = this;
        util.log('event:' + name + ',source:' + me.name);

        var list = me.events[name];
        if (list) {
            // Copy cb lists to prevent modification
            list = list.slice();

            // Execute event cbs, use index because it's the faster.
            for (var i = 0, len = list.length; i < len; i++) {
                list[i].call(me, data);
            }
        }

        return me;
    }
    Mod.prototype.resolveName = function (name) {
        //基于模块的base，计算uri
        var baseUrl = this.base;

        if (/^(http|https|file):\/\//.test(name)) {
            return name;
        } else if (baseUrl) {
            if (name == 'root') {
                return name;
            }
            //baseurl:http://localhost/sss/

            //../../js/..//../ss/./../p.js
            //如果模块名没有后缀，默认.js，
            name = normlizeName(name);
            if (name.substring(0, 1) == '/') {
                debugger;
                var reg_pro = /^(?:http|https|file):\/\/[^\/]*/;
                name = reg_pro.exec(baseUrl)[0] + name;
            } else {


                name = baseUrl + name;

                //aaa/./ad/aaa/ss.js=>ad/aaa/ss.js
                name = name.replace(/\/\.\//g, '/');

                //../../b/c.js=>bbb/aaa/../../b/c.js
                var regx = /\/[^/]+\/\.\.\//;
                while (name.match(regx)) {
                    name = name.replace(regx, "/")
                }
            }
            return name;
        }
        return name;
    }
    Mod.prototype.save = function () {
        var me = this;
        me.setLoading();
        // loadFile(me.name);
        //依赖检查,如果没有依赖，或依赖都加载完，本模块触发loaded事件
        if (!me.loadCheck()) {
            me.subLoad();
        }
        loader.save(me.uri, me.factory);
    }
    //订阅依赖模块的load事件
    Mod.prototype.subLoad = function () {
        var me = this;
        util.each(me.deps, function (dep, i) {
            //根据模块的base，计算模块的uri；
            var m = Mod.getOrCreate(dep);

            if (m.isLoaded()) {
                return;
            }

            util.log('loaded:' + m.name + ',parent:' + me.name);
            m.on('loaded', (function (parent) {
                return function () {
                    // util.log('loaded:' + m.name);
                    parent.loadCheck({ from: m });
                }
            })(me));

        });
        return me;
    }
    //补全依赖模块名称为uri
    Mod.prototype.normalizeDeps = function () {
        var me = this;
        var deps = [];
        if (me.deps) {
            util.each(me.deps, function (dep) {
                if (_cfx.externals[dep]) {
                    return;
                }
                dep = me.resolveName(dep);
                deps.push(dep);
                me.remain[dep] = true;
            });
        }
        me.deps = deps;
    }
    //加载依赖的模块，模块状态为init
    Mod.prototype.loadDeps = function () {
        var me = this;
        var ds = [];
        var depsType = {};
        util.each(me.deps, function (dep, i) {
            //根据模块的base，计算模块的uri；
            var m = Mod.getOrCreate(dep);
            if (m.isInit()) {
                util.log('init:' + m.name);
                m.setLoading();
                ds.push(m.uri);
            }
        });

        if (ds.length <= 0) {
            return;
        }

        loader.load(ds, function (err, deps) {
            if (err == true) {
                me.emit('loaderror', err);
            } else {
                //匿名加载
                if (_currentDefine && _currentDefine.deps) {
                    util.log('set module name :' + deps)
                    if (util.isArray(deps)) {
                        deps = deps[0];
                    }
                    _currentDefine.name = deps;
                    var def = _currentDefine;
                    _currentDefine = null;
                    var m = Mod.getOrCreate(def.name, {
                        deps: def.deps,
                        factory: def.factory
                    });
                    m.load();
                } else {
                    if (!deps) {
                        return;
                    }

                    if (!util.isArray(deps)) {
                        Mod.getOrCreate(deps).load();
                    } else {
                        util.each(deps, function (dep) {
                            Mod.getOrCreate(dep).load();
                        });
                    }
                }
            }
        })
    }
    //加载模块依赖，如果是打包模式，不需要执行
    Mod.prototype.load = function () {
        //加载模块，处理依赖的加载，递归
        //计算js的uri
        var me = this;
        //已经加载完，不需要重新加载
        if (me.isLoaded()) {
            return true;
        }

        // loadFile(me.name);
        //依赖检查,如果没有依赖，或依赖都加载完，本模块触发loaded事件
        if (!me.loadCheck()) {
            me.subLoad();
            me.loadDeps();
        }

        loader.save(me.uri, me.factory);
    }

    //load状态检查，返回是否已加载。
    Mod.prototype.loadCheck = function (opts) {
        var me = this;

        if (me.isLoaded()) {
            return true;
        }

        if (me.deps.length == 0) {
            me.setLoaded();
            return true;
        }

        var loaded = true;
        // var remains = {};
        util.each(me.remain, function (item, dep) {
            var m = Mod.getOrCreate(dep);
            if (!m) {
                loaded = false;
                return;
            }
            if (!m.isLoaded()) {
                loaded = false;
                return;
            }
            delete me.remain[dep];
        });

        if (loaded) {
            me.setLoaded();
        } else {
            if (_cfx.env != _ENV.production) {
                me.circelCheck();
            }
        }
        return loaded;
    }
    Mod.prototype.circelCheck = function () {
        var me = this;
        me.depsCount++;

        if (me.depsCount > 1) {
            me.depsCount = 0;
            throw new Error('circular reference:' + me.name);
        }

        util.each(me.deps, function (dep, i) {
            var m = Mod.getOrCreate(dep);
            if (m) {
                return m.circelCheck();
            }
        });

        me.depsCount--
    }
    Mod.prototype.require = function (name) {
        var me = this;

        if (_cfx.externals[name]) {
            return glb[_cfx.externals[name]];
        }
        var key = me.resolveName(name);
        var expt = _reg[key];
        if (!expt) {
            var m = (_map[key]);
            if (!m) {
                return {};
            }
            return m.exec();
        }
        return expt;
    }

    // Mod.prototype.exec = function () {
    //     var me = this;
    //     var expt = {};
    //     var exports = {};

    //     var require = util.bind(me.require, me);
    //     require.async = glb.require.async;
    //     me.setStatus(_STATUS.executing);
    //     var result = this.factory.apply(_context, [require, exports, this]);
    //     me.setStatus(_STATUS.executed);
    //     if (result) {
    //         expt = _reg[this.name] = result;
    //     } else {
    //         this.exports = util.assign(this.exports, exports);
    //         expt = _reg[this.name] = this.exports;
    //     }
    //     // delete me.factory;
    //     // delete me.remain;
    //     return expt
    // }

    Mod.prototype.exec = function () {
        var me = this;
        // var expt = {};
        // var exports = {};
        var result = '';
        // var require = util.bind(me.require, me);
        // require.async = glb.require.async;
        me.setStatus(_STATUS.executing);

        var lod = loaderManager.get(me.uri);

        _reg[me.name] = result = lod.exec(me); //this.factory.apply(_context, [require, exports, this]);

        me.setStatus(_STATUS.executed);

        // if (result) {
        //     expt = _reg[me.name] = result;
        // } else {
        //     me.exports = util.assign(this.exports, exports);
        //     expt = _reg[me.name] = this.exports;
        // }

        // me.exports = result;
        // delete me.factory;
        // delete me.remain;
        return result
    }

    Mod.prototype.setStatus = function (status, info) {
        var me = this;
        if (me.status !== status) {
            me.status = status;
            switch (status) {
                case _STATUS.loading:
                    me.emit('loading', me);
                    break;
                case _STATUS.loaded:
                    me.emit('loaded', me);
                    break;
                case _STATUS.executing:
                    me.emit('executing', me);
                    break;
                case _STATUS.executed:
                    me.emit('executed', me);
                    break;
                case _STATUS.error:
                    me.emit('error', info);
                    break;
                default:
                    break;
            }
        }
    }

    Mod.prototype.setLoaded = function () {
        this.setStatus(_STATUS.loaded);
    }

    Mod.prototype.setLoading = function () {
        this.setStatus(_STATUS.loading);
    }
    Mod.prototype.isLoaded = function () {
        return this.status == _STATUS.loaded
    }

    Mod.prototype.isLoading = function () {
        return this.status == _STATUS.loading
    }

    Mod.prototype.isInit = function () {
        return this.status == _STATUS.init;
    }

    Mod.get = function (uri, opts) {
        uri = parseName(uri);
        var m = _map[uri];
        return m;
    }

    Mod.getOrCreate = function (uri, opts) {
        opts = opts || {};
        uri = parseName(uri);

        var m = _map[uri];
        if (!m) {
            var cfx = util.assign({
                base: uri.replace(/[^\/]*$/, ''),
                name: uri,
                uri: uri
            }, opts);
            m = new Mod(cfx);
            m.normalizeDeps();
            _map[uri] = m;
        } else {
            m = util.assign(m, opts);
            m.normalizeDeps();
        }

        return m;
    }

    Mod.exist = function (uri) {
        return !!_map[uri];
    }

    Mod.getModule = function (uri) {
        var m = _map[uri];
        if (!m) {
            return {};
        }
        return m.exec();
    }

    return Mod;
})();
;
    var version = /**
 * 模块更新检查
 * 根据模块路径，查找该路径下是【模块名】.json文件，如果有，进行更新检查，没有不进行
 */
(function () {
    //检查之前缓存的pkg文件内容，如果整个的版本未变，则不需要更新
    //如果之前没有缓存，需要更新所有
    //cb(depsNeedUpdate)
    function checkPkg(pkg, cb) {
        if (!_cfx.localStore) {
            cb && cb(pkg.dependencies);
            return;
        }

        storage.get(pkg.name, function (val) {
            //不存在历史缓存，全部更新
            if (!val) {
                cb && cb(pkg.dependencies);
                return;
            }
            var oldPkg = typeof JSON == 'undefined' ? eval('(' + val + ')') : JSON.parse(val);
            var needs = {}
            if (oldPkg.version == pkg.version) {
                //主版本一致，不更新
                return cb && cb(null);
            } else {
                //主版本不一致，分析需要更新的依赖模块                
                util.each(pkg.dependencies, function (ver, pro) {
                    if (!oldPkg.dependencies[pro]) {
                        needs[pro] = ver;
                        return;
                    }
                    if (oldPkg.dependencies[pro] != ver) {
                        needs[pro] = ver;
                        return;
                    }
                })
            }
            cb && cb(needs);
        });
    }

    function check(pkg) {
        //判断本地缓存是否有map文件，如果有，检查模块是否存在，已经对比模块的版本，如果不存在或者版本有变化，下载对应的js模块
        var pkgs = pkg;

        if (util.isString(pkg)) {
            pkgs = typeof JSON == 'undefined' ? eval('(' + pkg + ')') : JSON.parse(pkg);
        }

        if (!util.isArray(pkgs)) {
            pkgs = [pkgs];
        }
        //先检查之前缓存的pkg文件，利用name

        util.each(pkgs, function (p, i) {

            checkPkg(p, function (depsNeedUpdate) {

                if (!depsNeedUpdate) {
                    return;
                }
                var deps = [];
                util.each(depsNeedUpdate, function (p, name) {
                    var uri = parseName(name);
                    deps.push(uri);
                    loader.remove(uri);
                    Mod.getOrCreate(uri).setLoading();                    
                });
                loader.update(deps, function (err, deps) {

                    if (_currentDefine && _currentDefine.deps) {
                        util.log('set module name :' + deps)
                        if (util.isArray(deps)) {
                            deps = deps[0];
                        }
                        _currentDefine.name = deps;
                        var def = _currentDefine;
                        _currentDefine = null;
                        var m = Mod.getOrCreate(def.name, {
                            deps: def.deps,
                            factory: def.factory
                        });
                        m.load();
                    } else {
                        if (!util.isArray(deps)) {
                            deps = [deps];
                        }
                        util.each(deps, function (dep) {
                            Mod.getOrCreate(dep).load();
                        });
                    }

                });
            });

            if (typeof JSON != 'undefined') {
                storage.set(p.name, JSON.stringify(p))
            }
        });
    }
    return {
        check: check
    }
})();

    function getRootId() {
        return 'ROOT' + (++_ROOT);
    }
    function commentReplace(match, singlePrefix) {
        return singlePrefix || '';
    }
    var interactiveScript;
    function getCurrentScript() {
        if (currentlyAddingScript) {
            return currentlyAddingScript
        }

        // For IE6-9 browsers, the script onload event may not fire right
        // after the script is evaluated. Kris Zyp found that it
        // could query the script nodes and the one that is in "interactive"
        // mode indicates the current script
        // ref: http://goo.gl/JHfFW
        if (interactiveScript && interactiveScript.readyState === "interactive") {
            return interactiveScript
        }

        var scripts = head.getElementsByTagName("script")

        for (var i = scripts.length - 1; i >= 0; i--) {
            var script = scripts[i]
            if (script.readyState === "interactive") {
                interactiveScript = script
                return interactiveScript
            }
        }
    }

    //分析模块名称，返回uri
    //abc||./abc=>abc.js
    //./js/aa=>
    function parseName(name) {

        var baseUrl = _cfx.baseUrl;

        if (/^(http|https|file):\/\//.test(name)) {
            return name;
        } else if (baseUrl) {
            if (name == 'root') {
                return name;
            }
            //baseurl:http://localhost/sss/

            //../../js/..//../ss/./../p.js
            //如果模块名没有后缀，默认.js，
            name = normlizeName(name);

            //绝对路径
            if (name.substring(0, 1) == '/') {  
                var reg_pro = /^(?:http|https|file):\/\/[^\/]*/;
                name=reg_pro.exec(baseUrl)[0]+name;
            } else {
                name = baseUrl + name;

                //aaa/./ad/aaa/ss.js=>ad/aaa/ss.js
                name = name.replace(/\/\.\//g, '/');

                //../../b/c.js=>bbb/aaa/../../b/c.js
                var regx = /\/[^/]+\/\.\.\//;
                while (name.match(regx)) {
                    name = name.replace(regx, "/")
                }
            }
            return name;
        }
        return name;
    }

    function removeBase(uri) {
        var baseUrl = _cfx.baseUrl;
        return uri.replace(baseUrl, './');
    }
    function normlizeName(name) {
        if (_cfx.ONLYJS) {
            return /\.js$/.test(name) ? name : name + '.js';
        }
        return /\.\w*$/g.test(name) ? name : name + '.js';
    }


    function define(name, deps, callback) {
        if (typeof name !== 'string') {
            callback = deps;
            deps = name;
            name = null;
        }
        if (!util.isArray(deps)) {
            callback = deps;
            deps = [];
        }
        //name命名，用script加载过程中，当前正在加载的uri。
        //Try to derive uri in IE6-9 for anonymous modules
        if (!name) {
            if (doc.attachEvent && typeof getCurrentScript !== "undefined") {
                var script = getCurrentScript()

                if (script) {
                    name = script.src
                }

                // NOTE: If the id-deriving methods above is failed, then falls back
                // to use onload event to get the uri
            }
            if (document.currentScript) {
                name = document.currentScript.src;
            }
        }
        var exports = {}, module = {};
        if (util.isFunction(callback)) {
            callback
                .toString()
                .replace(_commentRegExp, commentReplace)
                .replace(_cjsRequireRegExp, function (match, dep) {
                    deps.push(dep);
                });
            var def = { name: name, deps: deps, factory: callback };
            if (name) {
                var m = Mod.getOrCreate(def.name, {
                    deps: def.deps,
                    factory: def.factory
                })

                m.save();
            } else {
                _currentDefine = def;
            }
        }
    }


    function require(deps, callback) {
        //加载依赖
        if (!util.isArray(deps)) {
            callback = deps;
            deps = null;
        }
        if (util.isFunction(callback)) {

            if (!deps) {
                callback && callback();
                return;
            }

            var root = Mod.getOrCreate(getRootId(), {
                base: _cfx.baseUrl,
                deps: deps,
                factory: callback
            });

            root.on('loaded', function () {

                var m = this; params = [];
                util.each(m.deps, function (dep, i) {
                    params.push(Mod.getModule(dep));
                })
                callback && callback.apply(_context, params);
            });
            root.load();
        }
    }

    function async(deps, callback) {
        setTimeout(function () {
            if (util.isString(deps)) {
                deps = [deps];
            }

            require(deps, callback);
        }, 1);
    }

    function install(pkg) {
        _package = pkg;
    }
    glb.bondjs = {};
    glb.bondjs.config = function (config) {
        _cfx.baseUrl = config.baseUrl || _cfx.baseUrl;
        _cfx.env = config.env || 'production';//devlepment    
        _cfx.localStore = config.localStore;
        _cfx.comboHandler = config.comboHandler || _cfx.comboHandler;
        _cfx.externals = config.externals || _cfx.externals;
    }
    glb.bondjs.install = install;
    define.cmd = {
        jQuery: true
    }
    glb.define = define;
    glb.bondjs.use = function () {
        if (_package) {
            glb.bondjs.check(_package);
        }
        require.apply(glb, arguments);
    }
    glb.require = {};
    glb.require.async = async;
    glb.bondjs.check = version.check;
    glb.exports = {};
    glb.module = {};

    //default config
    glb.bondjs.config({
        baseUrl: util.getBaseUrl(),
        localStore: false,
        externals: {}
    });
    glb.bondjs.setStorage = function (store) {
        if (store) { storage = store; }
    }
})(window);